unit Unit1;

interface

uses
  Windows, Messages, ShellAPI, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ComCtrls;

type
  TInfoForm = class(TForm)
    OpenDialog: TOpenDialog;
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    FileOpenItem: TMenuItem;
    FileN1Item: TMenuItem;
    FileExitItem: TMenuItem;
    HelpMenu: TMenuItem;
    HelpAboutItem: TMenuItem;
    ImageList: TImageList;
    ListView: TListView;
    procedure FormCreate(Sender: TObject);
    procedure FileOpenItemClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FileExitItemClick(Sender: TObject);
  private
    procedure OpenRMD(const FileName: string);
    procedure WMDropFiles(var Message: TWMDropFiles); message WM_DROPFILES;
  public
    { Public declarations }
  end;

var
  InfoForm: TInfoForm;

implementation

uses
  VML_ZyXEL1496, VML_USRobotics, VML_Utils, VML_Consts;
  
{$R *.DFM}

procedure TInfoForm.OpenRMD(const FileName: string);
var
  Info: TRMDInfo;
  Error: Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    ListView.Items.Clear;
    { Form Caption }
    Caption := Format('RMD Info - %s', [FileName]);
    { Get RMD file info }
    Error := GetRMDFileInfo(FileName, Info);
    { Check error code }
    if Error = OK then
      { Success retrieving information }
      with ListView.Items, Info do
      begin
        with Add do
        begin
          Caption := 'ModemType';
          SubItems.Add(ModemType);
          SubItems.Add('Modem type');
        end;
        with Add do
        begin
          Caption := 'ModemName';
          SubItems.Add(ModemName);
          SubItems.Add('Modem name');
        end;
        with Add do
        begin
          Caption := 'Compression';
          SubItems.Add(IntToStr(Compression));
          SubItems.Add('Modem specific compression method');
        end;
        with Add do
        begin
          Caption := 'CompressionName';
          SubItems.Add(CompressionName);
          SubItems.Add('Compression name');
        end;
        with Add do
        begin
          Caption := 'Speed';
          SubItems.Add(IntToStr(Speed));
          SubItems.Add('Sample speed');
        end;
        with Add do
        begin
          Caption := 'Bits';
          SubItems.Add(IntToStr(Bits));
          SubItems.Add('Bits per sample');
        end;
        with Add do
        begin
          Caption := 'DataSize';
          SubItems.Add(IntToStr(DataSize));
          SubItems.Add('Voice data size in bytes');
        end;
        with Add do
        begin
          Caption := 'VoiceLen';
          SubItems.Add(IntToStr(VoiceLen));
          SubItems.Add('Voice message length in msecs');
        end;
      end
    else
      { An error occurred while retrieving RMD file info }
      with ListView.Items.Add do
      begin
        Caption := 'Error';
        SubItems.Add(IntToStr(Error));
        SubItems.Add('An error occurred while retrieving RMD file information');
      end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TInfoForm.WMDropFiles(var Message: TWMDropFiles);
var
  DropCount, DropSize: Word;
  DropName: string;
begin
  with Message do
  begin
    DropCount := DragQueryFile(Drop, $FFFFFFFF, nil, 0);
    if DropCount > 0 then
    begin
      DropSize := DragQueryFile(Drop, 0, nil, 0);
      SetLength(DropName, DropSize);
      DragQueryFile(Drop, 0, PChar(DropName), DropSize + 1);
      OpenRMD(DropName);
    end;
    DragFinish(Drop);
  end;
  inherited;
end;

procedure TInfoForm.FormCreate(Sender: TObject);
const
  LVM_SETEXTENDEDLISTVIEWSTYLE = $1036;
  LVS_EX_FULLROWSELECT         = $00000020;
begin
  { Set full row select mode for Listview }
  SendMessage(ListView.Handle, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, LVS_EX_FULLROWSELECT);
  { Allow file drop access }
  DragAcceptFiles(Handle, True);
end;

procedure TInfoForm.FormActivate(Sender: TObject);
begin
  Update;
  { Check file name in params }
  if ParamCount > 0 then
  begin
    OpenRMD(ParamStr(1));
    Exit;
  end;
  { Otherwise open file }
  FileOpenItem.Click;
end;

procedure TInfoForm.FileOpenItemClick(Sender: TObject);
begin
  if OpenDialog.Execute then OpenRMD(OpenDialog.FileName);
end;

procedure TInfoForm.FileExitItemClick(Sender: TObject);
begin
  Close;
end;

end.
