unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, CommPort, VML_Port, VML_Modem, VML_Consts,
  VML_USRobotics, VML_Profile, ComCtrls, VML_NoModem, VML_IS101, VML_Log;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Edit1: TEdit;
    Label2: TLabel;
    Edit2: TEdit;
    OpenButton: TButton;
    CloseButton: TButton;
    InitButton: TButton;
    CommandButton: TButton;
    ReadButton: TButton;
    WriteButton: TButton;
    FlushButton: TButton;
    VoiceButton: TButton;
    DataButton: TButton;
    RecordButton: TButton;
    PlayButton: TButton;
    StopButton: TButton;
    DTMFButton: TButton;
    ClearButton: TButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    Memo1: TMemo;
    TabSheet2: TTabSheet;
    Memo2: TMemo;
    TabSheet3: TTabSheet;
    Memo3: TMemo;
    TabSheet5: TTabSheet;
    Memo5: TMemo;
    VoiceModem1: TVoiceModem;
    ModemProfile1: TModemProfile;
    USRobotics1: TUSRobotics;
    ModemPort1: TModemPort;
    ModemLogFile1: TModemLogFile;
    ModemTraceFile1: TModemLogFile;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormResize(Sender: TObject);
    procedure OpenButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure InitButtonClick(Sender: TObject);
    procedure CommandButtonClick(Sender: TObject);
    procedure ReadButtonClick(Sender: TObject);
    procedure WriteButtonClick(Sender: TObject);
    procedure FlushButtonClick(Sender: TObject);
    procedure VoiceButtonClick(Sender: TObject);
    procedure DataButtonClick(Sender: TObject);
    procedure RecordButtonClick(Sender: TObject);
    procedure PlayButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure DTMFButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure VoiceModem1DebugMsg(Sender: TObject; Msg: Integer; const S: string);
    procedure VoiceModem1Event(Sender: TObject; Event: Integer; Data: TEventData; var Result: Integer);
    procedure VoiceModem1ReadChar(Sender: TObject; C: Char);
    procedure VoiceModem1Trace(Sender: TObject; const Action, Line: string);
    procedure VoiceModem1WriteChar(Sender: TObject; C: Char);
    procedure VoiceModem1WriteString(Sender: TObject; const S: string);
    procedure VoiceModem1Yield(Sender: TObject);
  private
    FTraceWasCR: Boolean;
    FTraceAction: string;
    FLastEvent: Integer;
    procedure LockModem(const Proc: string);
    procedure TraceChar(const Action: string; C: Char);
    procedure UnlockModem;
    procedure WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

function CharToCode(C: Char): string;
begin
  if C >= ' ' then
    Result := C
  else
    Result := '#' + IntToStr(Ord(C));
end;

function StrToCode(const S: string): string;
var
  I: Integer;
begin
  Result := '';
  for I := 1 to Length(S) do
    if S[I] >= ' ' then
      Result := Result + S[I]
    else
      Result := Result + '#' + IntToStr(Ord(S[I]));
end;

procedure TForm1.LockModem(const Proc: string);
begin
  if not VoiceModem1.Lock(Self) then
  begin
    Memo2.Lines.Add(Format('%s: FAIL (%d)', [Proc, VME_MODEM_LOCKED]));
    Abort;
  end;
  Screen.Cursor := crAppStart;
end;

procedure TForm1.TraceChar(const Action: string; C: Char);
var
  S, L: string;
begin
  { Check for CR and LF }
  case C of
    CR:
      begin
        FTraceWasCR := True;
        Exit;
      end;
    LF:
      if FTraceWasCR then
      begin
        if FTraceAction = Action then
        begin
          FTraceWasCR := False;
          S := CR + LF;
        end
        else
          Exit;
      end
      else
        S := CharToCode(C);
    else
      begin
        if FTraceWasCR then
        begin
          FTraceWasCR := False;
          if Memo3.Lines.Count > 0 then
          begin
            S := Memo3.Lines[Memo3.Lines.Count - 1];
            Memo3.Lines[Memo3.Lines.Count - 1] := S + CR + LF;
          end;
        end;
        S := CharToCode(C);
      end;
  end;
  { Add char to trace log }
  if Memo3.Lines.Count > 0 then
  begin
    L := Memo3.Lines[Memo3.Lines.Count - 1];
    if Length(L) = 0 then
      Memo3.Lines[Memo3.Lines.Count - 1] := S
    else if FTraceAction = Action then
      Memo3.Lines[Memo3.Lines.Count - 1] := L + S
    else
      Memo3.Lines.Add(S);
  end
  else
    Memo3.Lines.Add(S);
  { Save current action }
  FTraceAction := Action;
end;

procedure TForm1.UnLockModem;
begin
  Screen.Cursor := crDefault;
  VoiceModem1.Unlock;
end;

procedure TForm1.WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo);
begin
  inherited;
  with Message.MinMaxInfo^ do
  begin
    ptMinTrackSize.X := 350;
    ptMinTrackSize.Y := 462;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  with ModemLogFile1 do FileName := ExtractFilePath(Application.ExeName) + FileName;
  with ModemTraceFile1 do FileName := ExtractFilePath(Application.ExeName) + FileName;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  Screen.Cursor := crHourGlass;
  try
    LockModem('Quit');
    try
      if VoiceModem1.Opened then
      try
        if VoiceModem1.StopCurrentAction = FAIl then Exit;
        if VoiceModem1.SetDevice(NO_DEVICE) = FAIL then Exit;
        if VoiceModem1.HangUp = FAIL then Exit;
      finally
        VoiceModem1.CloseDevice;
      end;
    except
      { If no driver specified some function raise exception. It need to be
        ignored for properly closing form. }
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Edit1.Width := ClientWidth - 179;
  Edit2.Width := ClientWidth - 179;
  with OpenButton do Left := Self.ClientWidth - (Width + 13);
  with CloseButton do Left := Self.ClientWidth - (Width + 13);
  with InitButton do Left := Self.ClientWidth - (Width + 13);
  with CommandButton do Left := Self.ClientWidth - (Width + 13);
  with ReadButton do Left := Self.ClientWidth - (Width + 13);
  with WriteButton do Left := Self.ClientWidth - (Width + 13);
  with FlushButton do Left := Self.ClientWidth - (Width + 13);
  with VoiceButton do Left := Self.ClientWidth - (Width + 13);
  with DataButton do Left := Self.ClientWidth - (Width + 13);
  with RecordButton do Left := Self.ClientWidth - (Width + 13);
  with PlayButton do Left := Self.ClientWidth - (Width + 13);
  with StopButton do Left := Self.ClientWidth - (Width + 13);
  with DTMFButton do Left := Self.ClientWidth - (Width + 13);
  with ClearButton do Left := Self.ClientWidth - (Width + 13);
  with PageControl1 do
  begin
    Width := OpenButton.Left - Left - 17;
    Height := Self.ClientHeight - Top - 11;
  end;
end;

procedure TForm1.OpenButtonClick(Sender: TObject);
begin
  case VoiceModem1.OpenDevice of
    OK: Memo2.Lines.Add('Open: OK');
    FAIL: Memo2.Lines.Add(Format('Open: FAIL (%d)', [VoiceModem1.LastError]));
  end;
end;

procedure TForm1.CloseButtonClick(Sender: TObject);
begin
  case VoiceModem1.CloseDevice of
    OK: Memo2.Lines.Add('Close: OK');
    FAIL: Memo2.Lines.Add(Format('Close: FAIL (%d)', [VoiceModem1.LastError]));
  end;
end;

procedure TForm1.InitButtonClick(Sender: TObject);
begin
  LockModem('Init');
  try
    case VoiceModem1.Init of
      OK: Memo2.Lines.Add('Init: OK');
      FAIL: Memo2.Lines.Add(Format('Init: FAIL (%d)', [VoiceModem1.LastError]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.CommandButtonClick(Sender: TObject);
var
  Result: Integer;
begin
  LockModem('Command');
  try
    Result := VoiceModem1.Command(Edit1.Text, Edit2.Text);
    case  Result of
      VMA_FAIL: Memo2.Lines.Add(Format('Command: VMA_FAIL (%d)', [VoiceModem1.LastError]));
      else Memo2.Lines.Add(Format('Command: %s', [ModemAnswerToStr(Result)]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.ReadButtonClick(Sender: TObject);
var
  StrRead: string;
begin
  LockModem('Read');
  try
    case VoiceModem1.ReadStr(StrRead) of
      OK: Memo2.Lines.Add('Read: OK');
      FAIL: Memo2.Lines.Add(Format('Read: FAIL (%d)', [VoiceModem1.LastError]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.WriteButtonClick(Sender: TObject);
begin
  LockModem('Write');
  try
    case VoiceModem1.WriteStr(Edit1.Text) of
      OK: Memo2.Lines.Add('Write: OK');
      FAIL: Memo2.Lines.Add(Format('Write: FAIL (%d)', [VoiceModem1.LastError]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.FlushButtonClick(Sender: TObject);
begin
  LockModem('Flush');
  try
    case VoiceModem1.Flush of
      OK: Memo2.Lines.Add('Flush: OK');
      FAIL: Memo2.Lines.Add(Format('Flush: FAIL (%d)', [VoiceModem1.LastError]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.VoiceButtonClick(Sender: TObject);
begin
  LockModem('Voice');
  try
    case VoiceModem1.VoiceModeOn of
      OK: Memo2.Lines.Add('VoiceModeOn: OK');
      FAIL: Memo2.Lines.Add(Format('VoiceModeOn: FAIL (%d)', [VoiceModem1.LastError]));
    end;
    case VoiceModem1.InitVoice of
      OK: Memo2.Lines.Add('InitVoice: OK');
      FAIL: Memo2.Lines.Add(Format('InitVoice: FAIL (%d)', [VoiceModem1.LastError]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.DataButtonClick(Sender: TObject);
begin
  LockModem('Data');
  try
    case VoiceModem1.VoiceModeOff of
      OK: Memo2.Lines.Add('VoiceModeOff: OK');
      FAIL: Memo2.Lines.Add(Format('VoiceModeOff: FAIL (%d)', [VoiceModem1.LastError]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.RecordButtonClick(Sender: TObject);
begin
  LockModem('Record');
  try
    case VoiceModem1.SetDevice(ModemProfile1.RecDevice) of
      OK: Memo2.Lines.Add('SetDevice: OK');
      FAIL: Memo2.Lines.Add(Format('SetDevice: FAIL (%d)', [VoiceModem1.LastError]));
    end;
    case VoiceModem1.RecordFile(Edit1.Text) of
      OK: Memo2.Lines.Add(Format('Record: ''%s'', OK', [Edit1.Text]));
      FAIL: Memo2.Lines.Add(Format('Record: ''%s'', FAIL (%d)', [Edit1.Text, VoiceModem1.LastError]));
      INTERRUPTED: Memo2.Lines.Add(Format('Record: ''%s'', INTERRUPTED', [Edit1.Text]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.PlayButtonClick(Sender: TObject);
begin
  LockModem('Play');
  try
    case VoiceModem1.SetDevice(ModemProfile1.PlayDevice) of
      OK: Memo2.Lines.Add('SetDevice: OK');
      FAIL: Memo2.Lines.Add(Format('SetDevice: FAIL (%d)', [VoiceModem1.LastError]));
    end;
    case VoiceModem1.PlayFile(Edit1.Text) of
      OK: Memo2.Lines.Add(Format('Play: ''%s'', OK', [Edit1.Text]));
      FAIL: Memo2.Lines.Add(Format('Play: ''%s'', FAIL (%d)', [Edit1.Text, VoiceModem1.LastError]));
      INTERRUPTED: Memo2.Lines.Add(Format('Play: ''%s'', INTERRUPTED', [Edit1.Text]));
    end;
  finally
    UnLockModem;
  end;
end;

procedure TForm1.StopButtonClick(Sender: TObject);
begin
  VoiceModem1.Stop;
end;

procedure TForm1.DTMFButtonClick(Sender: TObject);
begin
  VoiceModem1.Driver.GetCompression('');
(*
  LockModem('Wait');
  try
    case VoiceModem1.SetDevice(ModemProfile1.PlayDevice) of
      OK: Memo2.Lines.Add('SetDevice: OK');
      FAIL: Memo2.Lines.Add(Format('SetDevice: FAIL (%d)', [VoiceModem1.LastError]));
    end;
    case VoiceModem1.PlayDTMF(Edit1.Text) of
      OK: Memo2.Lines.Add(Format('PlayDTMF: ''%s'', OK', [Edit1.Text]));
      FAIL: Memo2.Lines.Add(Format('PlayDTMF: ''%s'', FAIL (%d)', [Edit1.Text, VoiceModem1.LastError]));
    end;
  finally
    UnLockModem;
  end;
*)
end;

procedure TForm1.ClearButtonClick(Sender: TObject);
begin
  Memo2.Lines.Clear;
  Memo1.Lines.Clear;
  Memo3.Lines.Clear;
end;

procedure TForm1.VoiceModem1DebugMsg(Sender: TObject; Msg: Integer; const S: string);
begin
  Memo1.Lines.Add(StrToCode(S));
end;

procedure TForm1.VoiceModem1Event(Sender: TObject; Event: Integer;
  Data: TEventData; var Result: Integer);
begin
  FLastEvent := Event;
end;

procedure TForm1.VoiceModem1ReadChar(Sender: TObject; C: Char);
begin
  // TraceChar('Recv', C);
end;

procedure TForm1.VoiceModem1Trace(Sender: TObject; const Action, Line: string);
begin
  Memo3.Lines.Add(Action + ': ' + Line);
end;

procedure TForm1.VoiceModem1WriteChar(Sender: TObject; C: Char);
begin
  // TraceChar('Send', C);
end;

procedure TForm1.VoiceModem1WriteString(Sender: TObject; const S: string);
// var
  // I: Integer;
begin
  // for I := 1 to Length(S) do TraceChar('Send', S[I]);
end;

procedure TForm1.VoiceModem1Yield(Sender: TObject);
begin
  Application.ProcessMessages;
end;

end.


