{
  Voice Modem Library v1.0

  This file contain definition of common constants,
  types and routines.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Consts;

interface

uses                               
  SysUtils, Classes, VML_Idents;

const

{ Common constants }

  OK          = 0;
  FAIL        = -1;
  INTERRUPTED = $4D00;

  UNKNOWN_EVENT = -2;

{ Possible errors }

  VME_BASE = 0;

  VME_SUCCESS                = VME_BASE + 0;  { Operation completed successfully }

  VME_NOT_IMPLEMENTED        = VME_BASE + 1;  { Method not implemented }
  VME_TIMEOUT                = VME_BASE + 2;  { Operation timeout }
  VME_INTERRUPTED            = VME_BASE + 3;  { Operation interrupted }

  VME_EVENT_NOT_HANDLED      = VME_BASE + 4;  { Unqueued event not handled }
  VME_EVENT_QUEUE_FULL       = VME_BASE + 5;  { Event queue is full }
  VME_EVENT_QUEUE_EMPTY      = VME_BASE + 6;  { No more events in event queue }
  VME_EVENT_LOCKED           = VME_BASE + 7;  { Unquueued event is locked }

  VME_PROFILE_NOT_SPECIFIED  = VME_BASE + 8;  { Profile property not specified }
  VME_PORT_NOT_SPECIFIED     = VME_BASE + 9;  { Port property not specified }
  VME_DRIVER_NOT_SPECIFIED   = VME_BASE + 10; { Driver not specified }
  VME_MODEM_NOT_SPECIFIED    = VME_BASE + 11; { Modem name not specified }

  VME_PORT_BUSY              = VME_BASE + 12; { Port is already in use }
  VME_PORT_ERROR             = VME_BASE + 13; { Unexpected port error }
  VME_PORT_NOT_OPENED         = VME_BASE + 14; { Port not opened }

  VME_MODEM_LOCKED           = VME_BASE + 15; { Modem already locked }
  VME_INVALID_MODEM_ANSWER   = VME_BASE + 16; { Invalid modem answer }
  VME_INVALID_COMMAND_FORMAT = VME_BASE + 17; { Invalid command format }

  VME_ILLEGAL_COMPRESSION    = VME_BASE + 18; { Illegal compression method }
  VME_UNKNOWN_DEVICE         = VME_BASE + 19; { Unknown device }

  VME_STREAM_READ_ERR        = VME_BASE + 20; { Stream read error }
  VME_STREAM_WRITE_ERR       = VME_BASE + 21; { Stream write error }
  VME_BAD_STREAM_FORMAT      = VME_BASE + 22; { Bad voice data stream format }
  VME_FORMAT_NOT_SUPPORTED   = VME_BASE + 23; { Voice data unsupported by current modem device }

  VME_FILE_NOT_FOUND         = VME_BASE + 24; { Given file not found }
  VME_FILE_OPEN_ERROR        = VME_BASE + 25; { Could not open input file }
  VME_FILE_CREATE_ERROR      = VME_BASE + 26; { Could not create output file }

  VME_OPERATION_IN_PROGRESS  = VME_BASE + 27; { Another voice operation in progress }
  VME_NO_ACTIVE_OPERATION    = VME_BASE + 28; { No active voice operation }

  VME_CONVERT_LIB_LOCKED     = VME_BASE + 29; { Data converting library already locked }
  VME_DATA_CONVERT_FAILED    = VME_BASE + 30; { Data converting error }

  VME_FFT_INIT_FAILED        = VME_BASE + 31; { FFT init failed }
  VME_RECOGNITION_FAILED     = VME_BASE + 32; { Answer recognition failed }

  VME_INVALID_STR_FORMAT     = VME_BASE + 33; { Invalid string format }

{ Possible input or output devices }

  NO_DEVICE           = $0001;
  DIALUP_LINE         = $0002;
  EXTERNAL_MICROPHONE = $0003;
  INTERNAL_MICROPHONE = $0004;
  EXTERNAL_SPEAKER    = $0005;
  INTERNAL_SPEAKER    = $0006;
  LOCAL_HANDSET       = $0007;
  SPEAKER_PHONE       = $0008;
  SOUND_CARD          = $0009;

{ Voice modem answers }

{ The user defined ones }

  VMA_USER         = $1000;
  VMA_USER_1       = $1000;
  VMA_USER_2       = $1001;
  VMA_USER_3       = $1002;
  VMA_USER_4       = $1003;
  VMA_USER_5       = $1004;
  VMA_USER_6       = $1005;
  VMA_USER_7       = $1006;
  VMA_USER_8       = $1007;
  VMA_USER_9       = $1008;
  VMA_USER_10      = $1009;
  VMA_USER_11      = $100A;
  VMA_USER_12      = $100B;
  VMA_USER_13      = $100C;
  VMA_USER_14      = $100D;
  VMA_USER_15      = $100E;
  VMA_USER_16      = $100F;

{ The default ones }

  VMA_DEFAULT      = $2000;
  VMA_BUSY         = $2000;
  VMA_CONNECT      = $2001;
  VMA_EMPTY        = $2002;
  VMA_ERROR        = $2003;
  VMA_FAX          = $2004;
  VMA_FCON         = $2005;
  VMA_FCO          = $2006;
  VMA_NO_ANSWER    = $2007;
  VMA_NO_CARRIER   = $2008;
  VMA_NO_DIAL_TONE = $2009;
  VMA_OK           = $200A;
  VMA_RINGING      = $200B;
  VMA_RING_1       = $200C;
  VMA_RING_2       = $200D;
  VMA_RING_3       = $200E;
  VMA_RING_4       = $200F;
  VMA_RING_5       = $2010;
  VMA_RING         = $2011;
  VMA_VCON         = $2012;

{ If something goes wrong }

  VMA_FAIL         = $4000;

{ Possible port, voice modem and shell execution states }

  DIALING          = $0000;
  IDLE             = $0001;
  INITIALIZING     = $0002;
  OFF_LINE         = $0003;
  ON_LINE          = $0004;
  PLAYING          = $0005;
  RECORDING        = $0006;
  WAITING          = $0007;

{ The different answer modes }

  ANSWER_DATA  = 1;
  ANSWER_FAX   = 2;
  ANSWER_VOICE = 4;

{ Dummy empty event }

  NO_EVENT                 = $0000;

{ Possible event messages for the voice modem }

  VOICE_CODE_EVENT         = $1000;
  VOICE_ANSWER_PHONE       = $1000;
  VOICE_BEEP               = $1001;
  VOICE_DIAL               = $1002;
  VOICE_HANDLE_DLE         = $1003;

  VOICE_INIT               = $1004;
  VOICE_MESSAGE_LIGHT_ON   = $1005;
  VOICE_MESSAGE_LIGHT_OFF  = $1006;
  VOICE_MODE_OFF           = $1007;
  VOICE_MODE_ON            = $1008;
  VOICE_PLAY_FILE          = $1009;
  VOICE_RECORD_FILE        = $100A;
  VOICE_SET_COMPRESSION    = $100B;
  VOICE_SET_DEVICE         = $100C;
  VOICE_STOP_DIALING       = $100D;
  VOICE_STOP_PLAYING       = $100E;
  VOICE_STOP_RECORDING     = $100F;
  VOICE_STOP_WAITING       = $1010;
  VOICE_SWITCH_TO_DATA_FAX = $1011;
  VOICE_WAIT               = $1012;

{ Possible event messages from the voice modem }

  VOICE_MODEM_EVENT        = $2000;
  BONG_TONE                = $2000;
  BUSY_TONE                = $2001;
  CALL_WAITING             = $2002;
  DIAL_TONE                = $2003;
  DATA_CALLING_TONE        = $2004;
  DATA_OR_FAX_DETECTED     = $2005;
  FAX_CALLING_TONE         = $2006;
  HANDSET_ON_HOOK          = $2007;
  HANDSET_OFF_HOOK         = $2008;
  LOOP_BREAK               = $2009;
  LOOP_POLARITY_CHANGE     = $200A;
  NO_ANSWER                = $200B;
  NO_CARRIER               = $200C;
  NO_DIAL_TONE             = $200D;
  NO_VOICE_ENERGY          = $200E;
  RING_DETECTED            = $200F;
  RINGBACK_DETECTED        = $2010;
  RECEIVED_DTMF            = $2011;
  SILENCE_DETECTED         = $2012;
  SIT_TONE                 = $2013;
  TDD_DETECTED             = $2014;
  VOICE_DETECTED           = $2015;
  REMOTE_HANGUP            = $2016;

  OPERATION_TIMEOUT        = $2021;

{ Possible signal events }

  SIGNAL_EVENT             = $4000;
  RESET_WATCHDOG           = $4000;
  SIGNAL_SIGCHLD           = $4001;
  SIGNAL_SIGHUP            = $4002;
  SIGNAL_SIGINT            = $4003;
  SIGNAL_SIGPIPE           = $4004;
  SIGNAL_SIGQUIT           = $4005;
  SIGNAL_SIGTERM           = $4006;
  SIGNAL_SIGUSR1           = $4007;
  SIGNAL_SIGUSR2           = $4008;
  SIGNAL_STOP              = $4009;

{ Event queue limit }

  MAX_EVENTS = 2048;

{ ASCII control codes }

  NUL = ^@; { $00 null (end string) }
  SOH = ^A; { $01 start of heading }
  STX = ^B; { $02 start of text }
  ETX = ^C; { $03 end of text }
  EOT = ^D; { $04 end of trasmition }
  ENQ = ^E; { $05 enquiry }
  ACK = ^F; { $06 acknowledge }
  BEL = ^G; { $07 bell }
  BS  = ^H; { $08 backspace }
  HT  = ^I; { $09 horizontal tab (TAB ) }
  LF  = ^J; { $0A line feed }
  VT  = ^K; { $0B vertical tab }
  FF  = ^L; { $0C form feed }
  CR  = ^M; { $0D carriage return }
  SO  = ^N; { $0E shift out }
  SI  = ^O; { $0F shift in }
  DLE = ^P; { $10 data line escape }
  DC1 = ^Q; { $11 device control 1 (X-ON) }
  DC2 = ^R; { $12 device control 2 }
  DC3 = ^S; { $13 device control 3 (X-OFF) }
  DC4 = ^T; { $14 device control 4 }
  NAK = ^U; { $15 negative acknowledge }
  SYN = ^V; { $16 synchronous idle }
  ETB = ^W; { $17 end transmit block }
  CAN = ^X; { $18 cancel }
  EM  = ^Y; { $19 end of medium }
  SUB = ^Z; { $1A substitute }
  ESC = ^[; { $1B escape }
  FS  = ^\; { $1C file separator }
  GS  = ^]; { $1D group separator }
  RS  = ^^; { $1E record separator }
  US  = ^_; { $1F unit separator }

  XON  = DC1; { XON char }
  XOFF = DC3; { XOFF char }

{ Modem messages }

  L_MESG  = 1;
  L_NOISE = 2;
  L_WARN  = 3;
  L_ERROR = 4;
  L_JUNK  = 5;

{ Buffer length for commands, voice modem answers and so on }

  VOICE_BUF_LEN = 1024;

{ Header Magic }

  RMD_HEADER_MAGIC = 'RMD1';

{$I *.INC}

type

{ Defines the header for raw modem data }

  TRMDHeader = packed record
    Magic: array[0..3] of Char;       { header magic }
    ModemType: array[0..15] of Char;  { modem identifier }
    Compression: SmallInt;            { record compression mode }
    Speed: SmallInt;                  { record speed }
    Bits: Byte;                       { bits per second }
    Reserved: array[0..6] of Char;    { reserved }
  end;

{ Some common VCL events }

  TMessageEvent = procedure(Sender: TObject; Msg: Integer; const S: string) of object;
  TProgressEvent = procedure(Sender: TObject; Done: Longint) of object;

const

  { Possible modem answers }

  ModemAnswers: array[0..30] of TIdentMapEntry = (
   (Value: VMA_BUSY; Name: 'BUSY'),
   (Value: VMA_CONNECT; Name: 'CONNECT'),
   (Value: VMA_ERROR; Name: 'ERROR'),
   (Value: VMA_FAX; Name: 'FAX'),
   (Value: VMA_FCON; Name: '+FCON'),
   (Value: VMA_FCO; Name: '+FCO'),
   (Value: VMA_NO_ANSWER; Name: 'NO ANSWER'),
   (Value: VMA_NO_CARRIER; Name: 'NO CARRIER'),
   (Value: VMA_NO_DIAL_TONE; Name: 'NO DIALTONE'),
   (Value: VMA_NO_DIAL_TONE; Name: 'NO DIAL TONE'),
   (Value: VMA_OK; Name: 'OK'),
   (Value: VMA_RINGING; Name: 'RINGING'),
   (Value: VMA_RING_1; Name: 'RING1'),
   (Value: VMA_RING_2; Name: 'RING2'),
   (Value: VMA_RING_3; Name: 'RING3'),
   (Value: VMA_RING_4; Name: 'RING4'),
   (Value: VMA_RING_5; Name: 'RING5'),
   (Value: VMA_RING_1; Name: 'RING 1'),
   (Value: VMA_RING_2; Name: 'RING 2'),
   (Value: VMA_RING_3; Name: 'RING 3'),
   (Value: VMA_RING_4; Name: 'RING 4'),
   (Value: VMA_RING_5; Name: 'RING 5'),
   (Value: VMA_RING_1; Name: 'RING A'),
   (Value: VMA_RING_2; Name: 'RING B'),
   (Value: VMA_RING_3; Name: 'RING C'),
   (Value: VMA_RING_4; Name: 'RING D'),
   (Value: VMA_RING_5; Name: 'RING E'),
   (Value: VMA_RING; Name: 'RING'),
   (Value: VMA_VCON; Name: 'VCON'),
   (Value: VMA_VCON; Name: '#VCON'),
   (Value: VMA_VCON; Name: 'VOICE'));

  UserAnswers: array[0..15] of TIdentMapEntry = (
   (Value: VMA_USER; Name: 'VM_USER'),
   (Value: VMA_USER_2; Name: 'VM_USER_2'),
   (Value: VMA_USER_3; Name: 'VM_USER_3'),
   (Value: VMA_USER_4; Name: 'VM_USER_4'),
   (Value: VMA_USER_5; Name: 'VM_USER_5'),
   (Value: VMA_USER_6; Name: 'VM_USER_6'),
   (Value: VMA_USER_7; Name: 'VM_USER_7'),
   (Value: VMA_USER_8; Name: 'VM_USER_8'),
   (Value: VMA_USER_9; Name: 'VM_USER_9'),
   (Value: VMA_USER_10; Name: 'VM_USER_10'),
   (Value: VMA_USER_11; Name: 'VM_USER_11'),
   (Value: VMA_USER_12; Name: 'VM_USER_12'),
   (Value: VMA_USER_13; Name: 'VM_USER_13'),
   (Value: VMA_USER_14; Name: 'VM_USER_14'),
   (Value: VMA_USER_15; Name: 'VM_USER_15'),
   (Value: VMA_USER_16; Name: 'VM_USER_16'));

{ Some routines }

function StrToModemAnswer(const S: string): Longint;
function StrToModemAnswerDef(const S: string; const Default: Longint): Longint;

function ModemAnswerToStr(Answer: Longint): string;

implementation

{ Some routines }

function StrToModemAnswer(const S: string): Longint;
begin
  Result := StrToModemAnswerDef(S, VMA_FAIL);
end;

function StrToModemAnswerDef(const S: string; const Default: Longint): Longint;
begin
  { Check form empty answer }
  if Length(S) = 0 then
  begin
    Result := VMA_EMPTY;
    Exit;
  end;
  { Check form modem answers }
  if IdentToInt(S, Result, ModemAnswers) then Exit;
  { Check form user answers }
  if IdentToInt(S, Result, UserAnswers) then Exit;
  { Unknown modem answer }
  Result := Default;
end;

function ModemAnswerToStr(Answer: Longint): string;
begin
  { Check form modem answers }
  if IntToIdent(Answer, Result, ModemAnswers) then Exit;
  { Check form user answers }
  if IntToIdent(Answer, Result, UserAnswers) then Exit;
  { Check for VMA_FAIL value }
  if Answer = VMA_FAIL then
  begin
    Result := 'VMA_FAIL';
    Exit;
  end;
  { Unknown answer value }
  Result := SUnknownModemAnswer;
end;

end.
