{
  Voice Modem Library v1.0

  This file contain some idents conversion routines.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Idents;

interface

uses
  Windows, SysUtils, Classes;

{$IFDEF VER90 }

type
  TIdentMapEntry = record
    Value: Integer;
    Name: String;
  end;

function IdentToInt(const Ident: string; var Int: Longint; const Map: array of TIdentMapEntry): Boolean;
function IntToIdent(Int: Longint; var Ident: string; const Map: array of TIdentMapEntry): Boolean;

{$ENDIF}

type
  PIdentMap = ^TIdentMap;
  TIdentMap = array[0..255] of TIdentMapEntry;

function IdentToIntDef(const Ident: string; Default: Longint; const Map: array of TIdentMapEntry): Longint;
function IntToIdentDef(Int: Longint; const Default: string; const Map: array of TIdentMapEntry): string;

function GetIdentNameIndex(const Name: string; const Map: array of TIdentMapEntry): Integer;
function GetIdentNameIndexDef(const Name: string; DefIndex: Integer; const Map: array of TIdentMapEntry): Integer;

function GetIdentValueIndex(Value: Longint; const Map: array of TIdentMapEntry): Integer;
function GetIdentValueIndexDef(Value: Longint; DefIndex: Integer; const Map: array of TIdentMapEntry): Integer;

function GetIdentsList(const Map: array of TIdentMapEntry; Proc: TGetStrProc): Integer;
function GetIdentsStringList(const Map: array of TIdentMapEntry; Strings: TStrings): Integer;

implementation

{$IFDEF VER90}

function IdentToInt(const Ident: string; var Int: Longint; const Map: array of TIdentMapEntry): Boolean;
var
  I: Integer;
begin
  for I := Low(Map) to High(Map) do
    if CompareText(Map[I].Name, Ident) = 0 then
    begin
      Result := True;
      Int := Map[I].Value;
      Exit;
    end;
  Result := False;
end;

function IntToIdent(Int: Longint; var Ident: string; const Map: array of TIdentMapEntry): Boolean;
var
  I: Integer;
begin
  for I := Low(Map) to High(Map) do
    if Map[I].Value = Int then
    begin
      Result := True;
      Ident := Map[I].Name;
      Exit;
    end;
  Result := False;
end;

{$ENDIF}

function IdentToIntDef(const Ident: string; Default: Longint; const Map: array of TIdentMapEntry): Longint;
begin
  if not IdentToInt(Ident, Result, Map) then Result := Default;
end;

function IntToIdentDef(Int: Longint; const Default: string; const Map: array of TIdentMapEntry): string;
begin
  if not IntToIdent(Int, Result, Map) then Result := Default;
end;

function GetIdentNameIndex(const Name: string; const Map: array of TIdentMapEntry): Integer;
var
  I: Integer;
begin
  for I := Low(Map) to High(Map) do
    if CompareText(Map[I].Name, Name) = 0 then
    begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

function GetIdentNameIndexDef(const Name: string; DefIndex: Integer; const Map: array of TIdentMapEntry): Integer;
begin
  Result := GetIdentNameIndex(Name, Map);
  if Result = -1 then Result := DefIndex;
end;

function GetIdentValueIndex(Value: Longint; const Map: array of TIdentMapEntry): Integer;
var
  I: Integer;
begin
  for I := Low(Map) to High(Map) do
    if Map[I].Value = Value then
    begin
      Result := I;
      Exit;
    end;
  Result := -1;
end;

function GetIdentValueIndexDef(Value: Longint; DefIndex: Integer; const Map: array of TIdentMapEntry): Integer;
begin
  Result := GetIdentValueIndex(Value, Map);
  if Result = -1 then Result := DefIndex;
end;

function GetIdentsList(const Map: array of TIdentMapEntry; Proc: TGetStrProc): Integer;
var
  I: Integer;
begin
  Result := 0;
  for I := Low(Map) to High(Map) do
  begin
    Proc(Map[I].Name);
    Inc(Result);
  end;
end;

function GetIdentsStringList(const Map: array of TIdentMapEntry; Strings: TStrings): Integer;
type
  TGetStrFunc = function(const S: string): Integer of object;
var
  Proc: TGetStrFunc;
begin
  Proc := Strings.Add;
  Result := GetIdentsList(Map, TGetStrProc(Proc));
end;

end.

