{
  Voice Modem Library v1.0

  This file contains a dummy hardware driver.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_NoModem;

interface

uses
  Windows, Classes, VML_Modem, VML_Consts;

type

{ TNoModem }

  TNoModem = class(TModemDriver)
  public
    function AnswerPhone: Integer; override;
    function Beep(Frequency, Length: Integer): Integer; override;
    function Dial(const Number: string): Integer; override;
    function HandleDLE(Code: Char): Integer; override;
    function HandleEvent(Event: Integer; Data: TEventData): Integer; override;
    function HangUp: Integer; override;
    function Init: Integer; override;
    function InitVoice: Integer; override;
    class function ModemName: string; override;
    class function ModemRMDName: string; override;
    function PlayDTMF(const Number: string): Integer; override;
    function PlayStream(Stream: TStream; BPS: Integer): Integer; override;
    function RecordStream(Stream: TStream; BPS: Integer): Integer; override;
    function RestoreVoiceMode: Integer; override;
    function SetCompression(var Compression, Speed, Bits: Integer): Integer; override;
    function SetDevice(Device: Integer): Integer; override;
    function StartPlayStream: Integer; override;
    function StartRecordStream: Integer; override;
    function StopDialing: Integer; override;
    function StopPlayStream: Integer; override;
    function StopPlaying: Integer; override;
    function StopRecordStream: Integer; override;
    function StopRecording: Integer; override;
    function StopWaiting: Integer; override;
    function SwitchToDataFax(const Mode: string): Integer; override;
    function VoiceModeOff: Integer; override;
    function VoiceModeOn: Integer; override;
    function Wait(TimeOut: DWORD): Integer; override;
    function WaitAnswer(const ExpectedAnswers: string; TimeOut: DWORD): Integer; override;
  end;

implementation

{$I *.INC}

{ TNoModem }

function TNoModem.AnswerPhone: Integer;
begin
  DebugMsg(L_ERROR, SAnswerPhoneCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.Beep(Frequency, Length: Integer): Integer;
begin
  DebugMsg(L_ERROR, SBeepCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.Dial(const Number: string): Integer;
begin
  DebugMsg(L_ERROR, SDialCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.HandleDLE(Code: Char): Integer;
begin
  DebugMsg(L_ERROR, SHandleDLECall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.HandleEvent(Event: Integer; Data: TEventData): Integer;
begin
  DebugMsg(L_ERROR, SHandleEventCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.HangUp: Integer;
begin
  DebugMsg(L_ERROR, SHangUpCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.Init: Integer;
begin
  DebugMsg(L_ERROR, SInitCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.InitVoice: Integer;
begin
  DebugMsg(L_ERROR, SInitVoiceCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

class function TNoModem.ModemName: string;
begin
  Result := 'Serial Port';
end;

class function TNoModem.ModemRMDName: string;
begin
  Result := 'None';
end;

function TNoModem.PlayDTMF(const Number: string): Integer;
begin
  DebugMsg(L_ERROR, SPlayDTMFCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.PlayStream(Stream: TStream; BPS: Integer): Integer;
begin
  DebugMsg(L_ERROR, SPlayStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.RecordStream(Stream: TStream; BPS: Integer): Integer;
begin
  DebugMsg(L_ERROR, SRecordStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.RestoreVoiceMode: Integer;
begin
  DebugMsg(L_ERROR, SRestoreVoiceModeCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.SetCompression(var Compression, Speed, Bits: Integer): Integer;
begin
  DebugMsg(L_ERROR, SSetCompressionCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.SetDevice(Device: Integer): Integer;
begin
  DebugMsg(L_ERROR, SSetDeviceCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StartPlayStream: Integer;
begin
  DebugMsg(L_ERROR, SStartPlayStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StartRecordStream: Integer;
begin
  DebugMsg(L_ERROR, SStartRecordStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StopDialing: Integer;
begin
  DebugMsg(L_ERROR, SStopDialingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StopPlayStream: Integer;
begin
  DebugMsg(L_ERROR, SStopPlayStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StopPlaying: Integer;
begin
  DebugMsg(L_ERROR, SStopPlayingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StopRecordStream: Integer;
begin
  DebugMsg(L_ERROR, SStopRecordStreamCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StopRecording: Integer;
begin
  DebugMsg(L_ERROR, SStopRecordingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.StopWaiting: Integer;
begin
  DebugMsg(L_ERROR, SStopWaitingCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.SwitchToDataFax(const Mode: string): Integer;
begin
  DebugMsg(L_ERROR, SSwitchToDataFaxCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.VoiceModeOff: Integer;
begin
  DebugMsg(L_ERROR, SVoiceModeOffCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.VoiceModeOn: Integer;
begin
  DebugMsg(L_ERROR, SVoiceModeOnCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.Wait(TimeOut: DWORD): Integer;
begin
  DebugMsg(L_ERROR, SWaitCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

function TNoModem.WaitAnswer(const ExpectedAnswers: string; TimeOut: DWORD): Integer;
begin
  DebugMsg(L_ERROR, SWaitAnswerCall);
  Modem.LastError := VME_NOT_IMPLEMENTED;
  Result := FAIL;
end;

end.
