{
  Voice Modem Library v1.0

  This file contain definition of modem port class.

  Note: ModemPort component used CTS signal for transmit data from
  modem. Always set hardware flow control when modem initialized.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Port;

interface

uses
  Windows, SysUtils, Classes, CommPort, VML_Consts, VML_Profile, VML_Log;

type

{ TModemPort }

  {
    Modem port class

    Properties:

    Profile           - Modem profile. For properly working a port this
                        property must be always assigned.

    Events:

    OnDebugMsg         - Debug message event.
  }

  TModemPort = class(TCustomCommPort)
  private
    FProfile: TModemProfile;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    function Close: Integer;
    function Open: Integer;
    function Read(Buffer: Pointer; Count: Integer): Integer;
    function Write(Buffer: Pointer; Count: Integer): Integer;
  published
    property Profile: TModemProfile read FProfile write FProfile;
  end;

implementation

{ TModemPort }

procedure TModemPort.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FProfile) then FProfile := nil;
end;

function TModemPort.Close: Integer;
begin
  ClosePort;
  Result := OK;
end;

function TModemPort.Open: Integer;
begin
  { Check profile specified }
  if Profile = nil then
  begin
    Result := FAIL;
    Exit;
  end;
  { Init port params }
  case Profile.Port of
    1: ComNumber := cnCOM1;
    2: ComNumber := cnCOM2;
    3: ComNumber := cnCOM3;
    4: ComNumber := cnCOM4;
    else
      { Invalid port number }
      begin
        Result := FAIL;
        Exit;
      end;
  end;
  case Profile.PortBaudRate of
    CBR_110: BaudRate := br110;
    CBR_300: BaudRate := br300;
    CBR_600: BaudRate := br600;
    CBR_1200: BaudRate := br1200;
    CBR_2400: BaudRate := br2400;
    CBR_4800: BaudRate := br4800;
    CBR_9600: BaudRate := br9600;
    CBR_14400: BaudRate := br14400;
    CBR_19200: BaudRate := br19200;
    CBR_38400: BaudRate := br38400;
    CBR_56000: BaudRate := br56000;
    CBR_57600: BaudRate := br57600;
    CBR_115200: BaudRate := br115200;
    CBR_128000: BaudRate := br128000;
    CBR_256000: BaudRate := br256000;
    else
      { Invalid baud rate value }
      begin
        Result := FAIL;
        Exit;
      end;
  end;
  case Profile.PortDataBits of
    4: DataBits := db4;
    5: DataBits := db5;
    6: DataBits := db6;
    7: DataBits := db7;
    8: DataBits := db8;
    else
      { Invalid data bits value }
      begin
        Result := FAIL;
        Exit;
      end;
  end;
  case Profile.PortParity of
    NOPARITY: Parity := paNone;
    ODDPARITY: Parity := paOdd;
    EVENPARITY: Parity := paEven;
    MARKPARITY: Parity := paMark;
    SPACEPARITY: Parity := paSpace;
    else
      { Invalid parity value }
      begin
        Result := FAIL;
        Exit;
      end;
  end;
  case Profile.PortStopBits of
    ONESTOPBIT: StopBits := sb1_0;
    ONE5STOPBITS: StopBits := sb1_5;
    TWOSTOPBITS: StopBits := sb2_0;
    else
      { Invalid stop bits value }
      begin
        Result := FAIL;
        Exit;
      end;
  end;
  StatusDelay := Profile.PortStatusDelay;
  ReadDelay := Profile.PortReadDelay;
  WriteDelay := Profile.PortWriteDelay;
  InBuffSize := Profile.PortInBuffSize;
  OutBuffSize := Profile.PortOutBuffSize;
  { Open device }
  OpenPort;
  { Check result }
  if not Enabled then
  begin
    Result := FAIL;
    Exit;
  end;
  { Success }
  Result := OK;
end;

function TModemPort.Read(Buffer: Pointer; Count: Integer): Integer;
begin
  if not Enabled then
    Result := -1
  else if CharReady then
    Result := inherited GetBlock(Buffer^, Count)
  else
    Result := 0;
end;

function TModemPort.Write(Buffer: Pointer; Count: Integer): Integer;
begin
  if not Enabled then
    Result := -1
  else
    Result := inherited PutBlock(Buffer^, Count);
end;

end.
