{
  Voice Modem Library v1.0

  This file contain definition of voice profile class.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Profile;

interface

uses
  Windows, Classes, Registry, VML_Consts;

type

{
  Modem profile.

  Properties:

  Port              - Modem port number. The default value is 2 (i.e. COM2).
  PortBaudRate      - Baud rate at which the communications device operates.
                      The bps rate must be high enough for the compression
                      mode used. This must be set to 38400 for Rockell
                      modems. The default value is 38400. See CBR_XXX
                      constants list (Windows.pas) for more available
                      values.
  PortDataBits      - Number of bits in the bytes transmitted and received.
                      Default value is 8.
  PortParity        - Port parity scheme to be used. Default value is
                      NOPARITY. See XXXXPARITY constants list (Windows.pas)
                      for more available values.
  PortStopBits      - The number of stop bits to be used. Default value is
                      ONESTOPBIT. See XXXSTOPBIT constants list (Windows.pas)
                      for more available values.
  PortTimeOut       - Default port TimeOut in seconds for a read or write
                      modem operation. The default value is 10 seconds.
  PortStatusDelay   - Time in msec for the delay, when checking modem
                      status. A higher value will decrease machine load by
                      increasing reaction time. The default is 50 msec.
  PortReadDelay     - Time in msec for the delay, when no new data are
                      received from the modem. The default is 75 msec.
  PortWriteDelay    - Time in msec for the delay, when no new data are
                      sended to the modem. The default is 75 msec.
  PortInBuffSize    - Port input buffer size in bytes. The default value
                      is 4096 bytes.
  PortOutBuffSize   - Port output buffer size in bytes. The default value
                      is 4096 bytes.
  ModemName         - This is default modem name for selecting modem driver
                      from the list of registered drivers. The is no default
                      value for this property.
  ModemInitCmd      - Additional modem initialization command. Executed after
                      performing all standard and driver specific commands.
                      NOTE: answer of this command is not processed (skipped).
                      WARNING: do nothing to change local echo and result
                      codes settings.
  ModemInitAnsw     - Modem answer for additional modem initialization
                      command. 
  ModemLogging      - Enable or disable logging (or tracing) of commands of
                      modem. The default value is False.
  PlayMaxLen        - Maximum playing length in seconds. There is no limit
                      by default.
  PlayDevice        - Voice playing device. The default is EXTERNAL_SPEAKER.
  PlayFileName      - Default outgoing voice message file name.
  PlayPreview       - Play preview mode. This feature is turned off by default.
  PlayPreviewLen    - Maximum playing length for preview mode in seconds. Some
                      as PlayMaxLen but turned on by PlayPreview value. Default
                      is 10 seconds.
  RecCompression    - This is the default compression mode for recording
                      voice data. The mode 0 is a special mode, that will
                      automatically choose a sane default value for every
                      modem. The default is 0.
  RecSpeed          - This is the default voice data recording speed. It
                      is the number of samples per second. The speed 0 is
                      a special speed, that will automatically choose a
                      sane default value for every modem. The default is 0.
  RecSilenceLen     - Silence detection length in 0.1 seconds. If the modem
                      detects silence for this time, it sends a silence
                      detect to the host. Default is 2 seconds (20 * 0.1
                      seconds).
  RecSilenceSens    - Silence detection threshold in percent (0% to 100%).
                      Increase this value if you have a noisy phone line
                      and the silence detection doesn't work reliably. The
                      default is 40%.
  RecRemoveSilence  - If RecRemoveSilence is enabled, the trailing silence
                      of an incoming voice message as detected by the modem
                      will be deleted. This might cause you to miss parts
                      of a message if the silence threshold is high and the
                      caller is talking very quietly. To be on the safe
                      side, don't define this. This feature is turned off
                      by default.
  RecIgnoreSilence  - If RecIgnoreSilence is enabled, the trailing silence
                      of an incoming voice message as detected by the modem
                      will be ignored. Otherwise the recording will be
                      stopped. This feature is turned off by default.
  RecDevice         - Voice recording device. The default is
                      EXTERNAL_MICROPHONE.
  RecMaxLen         - Maximum recording length in seconds. Hang up if
                      somebody talks longer than this. Default is 5 minutes
                      (300 seconds).
  TransmitGain      - Some modems support setting the transmit gain
                      (Sens of the microphone in speaker phone mode).
                      This value can be set in percent (0% to 100%).
                      0% is off, 100% is maximum. To use the modem default
                      value set this to -1. The default is -1.
  ReceiveGain       - Some modems support setting the receive gain (speaker
                      volume in speaker phone mode). This value can be set
                      in percent (0% to 100%). 0% is off, 100% is maximum.
                      To use the modem default value set this to -1. The
                      default is -1.
  DTMFLen           - Minimum length of detected DTMF tones, in milliseconds.
                      This is currently only supported by ZyXel modems with a
                      ROM release of 6.12 or above. The default is 30
                      milliseconds.
  DTMFSens          - DTMF tone detection threshold in percent (0% to 100%).
                      Increase this if the modem erroneously detects DTMF tones,
                      decrease it if it fails to * detect real ones. This is
                      currently only supported by ZyXel modems with a ROM
                      release of 6.12 or above. The default is 40%.
  DTMFWait          - Time to wait for a DTMF tone to arrive when recording or
                      waiting for DTMF input in seconds. The default is to wait
                      for 7 seconds.
  BeepFrequency     - Frequency for the beep command in Hz. The default is 933Hz.
  BeepLength        - Length for the beep command in 0.001 seconds.
                      The default is 1.5 seconds (1500 * 0.001 seconds).
  CommandDelay      - Delay before sending a new voice command to the modem
                      in milliseconds. The default is 100 milliseconds.
  CommandEcho       - Usually command echo from the modem should be enabled.
                      Since some modems sometimes forget this echo, it is
                      disabled by default. Turning this option off makes
                      things more reliable, but bugs are much harder to
                      trace. The default is to disable command echo.
  DialTimeOut       - Default TimeOut for a dialout in seconds. The default
                      value is 90 seconds.
  DialMode          - Default dial mode (pulse or tone). The default value is
                      P (pulse).
  AnswerRecognition - Some modems can not correct to decide that a dialout
                      call has been answered. For this modems may be used
                      software recognition of voice answer of abonent. Set
                      this value TRUE to enable software answer detection.
                      The default is FALSE (hardware detection of answer).
                      NOTE: This feature not implemented in version 1.0.
  RingbackGoesAway  - Timeout for deciding that a dialout call has been
                      answered. If more than this many 1/10ths of a second
                      have passed since the last ringback, the modem assumes
                      the call has been answered. Default is 45 (4.5 sec).
  RingbackNeverCame - Timeout for deciding that a dialout call has been
                      answered. If more than this many 1/10ths of a second
                      have passed without any ringback the modem assumes the
                      call has been answered. Default is 100 (10 sec).
  RawData           - Output recorded voice samples without header and expect
                      raw voice data on input for playback.
                      This feature is turned off by default.
 }

  TModemProfile = class(TComponent)
  private
    FPort: Integer;
    FPortBaudRate: Integer;
    FPortDataBits: Integer;
    FPortParity: Integer;
    FPortStopBits: Integer;
    FPortTimeOut: Integer;
    FPortStatusDelay: Integer;
    FPortReadDelay: Integer;
    FPortWriteDelay: Integer;
    FPortInBuffSize: Integer;
    FPortOutBuffSize: Integer;
    FModemName: string;
    FModemInitCmd: string;
    FModemInitAnsw: string;
    FModemLogging: Boolean;
    FPlayMaxLen: Integer;
    FPlayDevice: Integer;
    FPlayFileName: string;
    FPlayPreview: Boolean;
    FPlayPreviewLen: Integer;
    FRecCompression: Integer;
    FRecSpeed: Integer;
    FRecSilenceLen: Integer;
    FRecSilenceSens: Integer;
    FRecRemoveSilence: Boolean;
    FRecIgnoreSilence: Boolean;
    FRecDevice: Integer;
    FRecMaxLen: Integer;
    FTransmitGain: Integer;
    FReceiveGain: Integer;
    FDTMFLen: Integer;
    FDTMFSens: Integer;
    FDTMFWait: Integer;
    FBeepFrequency: Integer;
    FBeepLength: Integer;
    FCommandDelay: Integer;
    FCommandEcho: Boolean;
    FDialTimeOut: Integer;
    FDialMode: string;
    FAnswerRecognition: Boolean;
    FRingbackGoesAway: Integer;
    FRingbackNeverCame: Integer;
    FRawData: Boolean;
    function IsModemInitAnswStored: Boolean;
  protected
    procedure AssignTo(Dest: TPersistent); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure LoadFromRegistry(Root: Integer; const Key: string); virtual;
    procedure SaveToRegistry(Root: Integer; const Key: string); virtual;
  published
    property Port: Integer read FPort write FPort default 2;
    property PortBaudRate: Integer read FPortBaudRate write FPortBaudRate default 38400;
    property PortParity: Integer read FPortParity write FPortParity default NOPARITY;
    property PortDataBits: Integer read FPortDataBits write FPortDataBits default 8;
    property PortStopBits: Integer read FPortStopBits write FPortStopBits default ONESTOPBIT;
    property PortTimeOut: Integer read FPortTimeOut write FPortTimeOut default 10;
    property PortStatusDelay: Integer read FPortStatusDelay write FPortStatusDelay default 50;
    property PortReadDelay: Integer read FPortReadDelay write FPortReadDelay default 75;
    property PortWriteDelay: Integer read FPortWriteDelay write FPortWriteDelay default 75;
    property PortInBuffSize: Integer read FPortInBuffSize write FPortInBuffSize default 4096;
    property PortOutBuffSize: Integer read FPortOutBuffSize write FPortOutBuffSize default 4096;
    property ModemName: string read FModemName write FModemName;
    property ModemInitCmd: string read FModemInitCmd write FModemInitCmd;
    property ModemInitAnsw: string read FModemInitAnsw write FModemInitAnsw stored IsModemInitAnswStored;
    property ModemLogging: Boolean read FModemLogging write FModemLogging default False;
    property PlayDevice: Integer read FPlayDevice write FPlayDevice default EXTERNAL_SPEAKER;
    property PlayMaxLen: Integer read FPlayMaxLen write FPlayMaxLen default 0;
    property PlayFileName: string read FPlayFileName write FPlayFileName;
    property PlayPreview: Boolean read FPlayPreview write FPlayPreview default False;
    property PlayPreviewLen: Integer read FPlayPreviewLen write FPlayPreviewLen default 10;
    property RecCompression: Integer read FRecCompression write FRecCompression default 0;
    property RecSpeed: Integer read FRecSpeed write FRecSpeed default 0;
    property RecSilenceLen: Integer read FRecSilenceLen write FRecSilenceLen default 20;
    property RecSilenceSens: Integer read FRecSilenceSens write FRecSilenceSens default 40;
    property RecRemoveSilence: Boolean read FRecRemoveSilence write FRecRemoveSilence default False;
    property RecIgnoreSilence: Boolean read FRecIgnoreSilence write FRecIgnoreSilence default False;
    property RecDevice: Integer read FRecDevice write FRecDevice default EXTERNAL_MICROPHONE;
    property RecMaxLen: Integer read FRecMaxLen write FRecMaxLen default 300;
    property TransmitGain: Integer read FTransmitGain write FTransmitGain default -1;
    property ReceiveGain: Integer read FReceiveGain write FReceiveGain default -1;
    property DTMFLen: Integer read FDTMFLen write FDTMFLen default 30;
    property DTMFSens: Integer read FDTMFSens write FDTMFSens default 40;
    property DTMFWait: Integer read FDTMFWait write FDTMFWait default 7;
    property BeepFrequency: Integer read FBeepFrequency write FBeepFrequency default 933;
    property BeepLength: Integer read FBeepLength write FBeepLength default 1500;
    property CommandDelay: Integer read FCommandDelay write FCommandDelay default 100;
    property CommandEcho: Boolean read FCommandEcho write FCommandEcho default False;
    property DialTimeOut: Integer read FDialTimeOut write FDialTimeOut default 90;
    property DialMode: string read FDialMode write FDialMode;
    property AnswerRecognition: Boolean read FAnswerRecognition write FAnswerRecognition default False;
    property RingbackGoesAway: Integer read FRingbackGoesAway write FRingbackGoesAway default 45;
    property RingbackNeverCame: Integer read FRingbackNeverCame write FRingbackNeverCame default 120;
    property RawData: Boolean read FRawData write FRawData default False;
  end;

implementation

{ TModemProfile }

constructor TModemProfile.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPort := 2;
  FPortBaudRate := 38400;
  FPortDataBits := 8;
  FPortParity := NOPARITY;
  FPortStopBits := ONESTOPBIT;
  FPortTimeOut := 10;
  FPortStatusDelay := 50;
  FPortReadDelay := 75;
  FPortWriteDelay := 75;
  FPortInBuffSize := 4096;
  FPortOutBuffSize := 4096;
  FModemName := '';
  FModemInitCmd := '';
  FModemInitAnsw := 'OK';
  FModemLogging := False;
  FPlayDevice := EXTERNAL_SPEAKER;
  FPlayMaxLen := 0;
  FPlayFileName := '';
  FPlayPreviewLen := 10;
  FRecCompression := 0;
  FRecSpeed := 0;
  FRecIgnoreSilence := False;
  FRecSilenceLen := 20;
  FRecSilenceSens := 40;
  FRecDevice := EXTERNAL_MICROPHONE;
  FRecMaxLen := 300;
  FTransmitGain := -1;
  FReceiveGain := -1;
  FDTMFLen := 30;
  FDTMFSens := 40;
  FDTMFWait := 7;
  FBeepFrequency := 933;
  FBeepLength := 1500;
  FCommandDelay := 100;
  FCommandEcho := False;
  FDialTimeOut := 90;
  FDialMode := 'P';
  FAnswerRecognition := False;
  FRingbackGoesAway := 45;
  FRingbackNeverCame := 120;
  FRawData := False;
end;

function TModemProfile.IsModemInitAnswStored: Boolean;
begin
  Result := FModemInitAnsw <> 'OK';
end;

procedure TModemProfile.AssignTo(Dest: TPersistent);
begin
  if Dest is TModemProfile then
  begin
    TModemProfile(Dest).Port := Port;
    TModemProfile(Dest).PortBaudRate := PortBaudRate;
    TModemProfile(Dest).PortDataBits := PortDataBits;
    TModemProfile(Dest).PortParity := PortParity;
    TModemProfile(Dest).PortStopBits := PortStopBits;
    TModemProfile(Dest).PortTimeOut := PortTimeOut;
    TModemProfile(Dest).PortStatusDelay := PortStatusDelay;
    TModemProfile(Dest).PortReadDelay := PortReadDelay;
    TModemProfile(Dest).PortWriteDelay := PortWriteDelay;
    TModemProfile(Dest).PortInBuffSize := PortInBuffSize;
    TModemProfile(Dest).PortOutBuffSize := PortOutBuffSize;
    TModemProfile(Dest).ModemName := ModemName;
    TModemProfile(Dest).ModemInitCmd := ModemInitCmd;
    TModemProfile(Dest).ModemInitAnsw := ModemInitAnsw;
    TModemProfile(Dest).ModemLogging := ModemLogging;
    TModemProfile(Dest).PlayDevice := PlayDevice;
    TModemProfile(Dest).PlayMaxLen := PlayMaxLen;
    TModemProfile(Dest).PlayFileName := PlayFileName;
    TModemProfile(Dest).PlayPreview := PlayPreview;
    TModemProfile(Dest).PlayPreviewLen := PlayPreviewLen;
    TModemProfile(Dest).RecCompression := RecCompression;
    TModemProfile(Dest).RecSpeed := RecSpeed;
    TModemProfile(Dest).RecSilenceLen := RecSilenceLen;
    TModemProfile(Dest).RecSilenceSens := RecSilenceSens;
    TModemProfile(Dest).RecIgnoreSilence := RecIgnoreSilence;
    TModemProfile(Dest).RecDevice := RecDevice;
    TModemProfile(Dest).RecMaxLen := RecMaxLen;
    TModemProfile(Dest).TransmitGain := TransmitGain;
    TModemProfile(Dest).ReceiveGain := ReceiveGain;
    TModemProfile(Dest).DTMFLen := DTMFLen;
    TModemProfile(Dest).DTMFSens := DTMFSens;
    TModemProfile(Dest).DTMFWait := DTMFWait;
    TModemProfile(Dest).BeepFrequency := BeepFrequency;
    TModemProfile(Dest).BeepLength := BeepLength;
    TModemProfile(Dest).CommandDelay := CommandDelay;
    TModemProfile(Dest).CommandEcho := CommandEcho;
    TModemProfile(Dest).DialTimeOut := DialTimeOut;
    TModemProfile(Dest).DialMode := DialMode;
    TModemProfile(Dest).AnswerRecognition := AnswerRecognition;
    TModemProfile(Dest).RingbackGoesAway := RingbackGoesAway;
    TModemProfile(Dest).RingbackNeverCame := RingbackNeverCame;
    TModemProfile(Dest).RawData := RawData;
    Exit;
  end;
  inherited AssignTo(Dest);
end;

procedure TModemProfile.LoadFromRegistry(Root: Integer; const Key: string);
begin
  with TRegIniFile.Create('') do
  begin
    { Open key }
    RootKey := Root;
    if OpenKey(Key, False) then
    begin
      { Read data }
      Port := ReadInteger('', 'Port', Port);
      PortBaudRate := ReadInteger('', 'PortBaudRate', PortBaudRate);
      PortParity := ReadInteger('', 'PortParity', PortParity);
      PortDataBits := ReadInteger('', 'PortDataBits', PortDataBits);
      PortStopBits := ReadInteger('', 'PortStopBits', PortStopBits);
      PortTimeOut := ReadInteger('', 'PortTimeOut', PortTimeOut);
      PortStatusDelay := ReadInteger('', 'PortStatusDelay', PortStatusDelay);
      PortReadDelay := ReadInteger('', 'PortReadDelay', PortReadDelay);
      PortWriteDelay := ReadInteger('', 'PortWriteDelay', PortWriteDelay);
      PortInBuffSize := ReadInteger('', 'PortInBuffSize', PortInBuffSize);
      PortOutBuffSize := ReadInteger('', 'PortOutBuffSize', PortOutBuffSize);
      ModemName := ReadString('', 'ModemName', ModemName);
      ModemInitCmd := ReadString('', 'ModemInitCmd', ModemInitCmd);
      ModemInitAnsw := ReadString('', 'ModemInitAnsw', ModemInitAnsw);
      ModemLogging := ReadBool('', 'ModemLogging', ModemLogging);
      PlayDevice := ReadInteger('', 'PlayDevice', PlayDevice);
      PlayMaxLen := ReadInteger('', 'PlayMaxLen', PlayMaxLen);
      PlayFileName := ReadString('', 'PlayFileName', PlayFileName);
      PlayPreview := ReadBool('', 'PlayPreview', PlayPreview);
      PlayPreviewLen := ReadInteger('', 'PlayPreviewLen', PlayPreviewLen);
      RecCompression := ReadInteger('', 'RecCompression', RecCompression);
      RecSpeed := ReadInteger('', 'RecSpeed', RecSpeed);
      RecSilenceLen := ReadInteger('', 'RecSilenceLen', RecSilenceLen);
      RecSilenceSens := ReadInteger('', 'RecSilenceSens', RecSilenceSens);
      RecIgnoreSilence := ReadBool('', 'RecIgnoreSilence', RecIgnoreSilence);
      RecDevice := ReadInteger('', 'RecDevice', RecDevice);
      RecMaxLen := ReadInteger('', 'RecMaxLen', RecMaxLen);
      TransmitGain := ReadInteger('', 'TransmitGain', TransmitGain);
      ReceiveGain := ReadInteger('', 'ReceiveGain', ReceiveGain);
      DTMFLen := ReadInteger('', 'DTMFLen', DTMFLen);
      DTMFSens := ReadInteger('', 'DTMFSens', DTMFSens);
      DTMFWait := ReadInteger('', 'DTMFWait', DTMFWait);
      BeepFrequency := ReadInteger('', 'BeepFrequency', BeepFrequency);
      BeepLength := ReadInteger('', 'BeepLength', BeepLength);
      CommandDelay := ReadInteger('', 'CommandDelay', CommandDelay);
      CommandEcho := ReadBool('', 'CommandEcho', CommandEcho);
      DialTimeOut := ReadInteger('', 'DialTimeOut', DialTimeOut);
      DialMode := ReadString('', 'DialMode', DialMode);
      AnswerRecognition := ReadBool('', 'AnswerRecognition', AnswerRecognition);
      RingbackGoesAway := ReadInteger('', 'RingbackGoesAway', RingbackGoesAway);
      RingbackNeverCame := ReadInteger('', 'RingbackNeverCame', RingbackNeverCame);
      RawData := ReadBool('', 'RawData', RawData);
    end;
  end;
end;

procedure TModemProfile.SaveToRegistry(Root: Integer; const Key: string);
begin
  with TRegIniFile.Create('') do
  begin
    { Open key }
    RootKey := Root;
    if OpenKey(Key, True) then
    begin
      { Write data }
      WriteInteger('', 'Port', Port);
      WriteInteger('', 'PortBaudRate', PortBaudRate);
      WriteInteger('', 'PortParity', PortParity);
      WriteInteger('', 'PortDataBits', PortDataBits);
      WriteInteger('', 'PortStopBits', PortStopBits);
      WriteInteger('', 'PortTimeOut', PortTimeOut);
      WriteInteger('', 'PortStatusDelay', PortStatusDelay);
      WriteInteger('', 'PortReadDelay', PortReadDelay);
      WriteInteger('', 'PortWriteDelay', PortWriteDelay);
      WriteInteger('', 'PortInBuffSize', PortInBuffSize);
      WriteInteger('', 'PortOutBuffSize', PortOutBuffSize);
      WriteString('', 'ModemName', ModemName);
      WriteString('', 'ModemInitCmd', ModemInitCmd);
      WriteString('', 'ModemInitAnsw', ModemInitAnsw);
      WriteBool('', 'ModemLogging', ModemLogging);
      WriteInteger('', 'PlayDevice', PlayDevice);
      WriteInteger('', 'PlayMaxLen', PlayMaxLen);
      WriteString('', 'PlayFileName', PlayFileName);
      WriteBool('', 'PlayPreview', PlayPreview);
      WriteInteger('', 'PlayPreviewLen', PlayPreviewLen);
      WriteInteger('', 'RecCompression', RecCompression);
      WriteInteger('', 'RecSpeed', RecSpeed);
      WriteInteger('', 'RecSilenceLen', RecSilenceLen);
      WriteInteger('', 'RecSilenceSens', RecSilenceSens);
      WriteBool('', 'RecIgnoreSilence', RecIgnoreSilence);
      WriteInteger('', 'RecDevice', RecDevice);
      WriteInteger('', 'RecMaxLen', RecMaxLen);
      WriteInteger('', 'TransmitGain', TransmitGain);
      WriteInteger('', 'ReceiveGain', ReceiveGain);
      WriteInteger('', 'DTMFLen', DTMFLen);
      WriteInteger('', 'DTMFSens', DTMFSens);
      WriteInteger('', 'DTMFWait', DTMFWait);
      WriteInteger('', 'BeepFrequency', BeepFrequency);
      WriteInteger('', 'BeepLength', BeepLength);
      WriteInteger('', 'CommandDelay', CommandDelay);
      WriteBool('', 'CommandEcho', CommandEcho);
      WriteInteger('', 'DialTimeOut', DialTimeOut);
      WriteString('', 'DialMode', DialMode);
      WriteBool('', 'AnswerRecognition', AnswerRecognition);
      WriteInteger('', 'RingbackGoesAway', RingbackGoesAway);
      WriteInteger('', 'RingbackNeverCame', RingbackNeverCame);
      WriteBool('', 'RawData', RawData);
    end;
  end;
end;

end.
