{
  Voice Modem Library v1.0

  This file contain Delphi VCL registration routines.

  Note: Some modem commands contain special control chars that incorrectly
  displayed by Object Inspector. These chars must be entered in one of
  following formats: a) <NAME> where NAME is name of ASCII Control Char
  (for example <DLE>, <CR>, <LF> and etc.) b) using control char mnemonic
  (for example ^C, ^M, ^J and etc.). During saving form resource it commands
  will be converted by property editor in line of control codes. For change
  these commands as decimal values click right mouse button on form, choose
  "View as Text" command and edit corresponding properties in Delphi IDE
  editor.

  Roman M. Mochalov
  E-mail: roman@sar.nnov.ru
}

unit VML_Reg;

interface

uses
  Windows, SysUtils, Classes, Dialogs, DsgnIntf;

type

{ TIntListProperty }

  TIntListProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
  end;

{ Port property }

  TPortProperty = class(TIntListProperty)
  public
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TBaudRateProperty }

  TBaudRateProperty = class(TIntListProperty)
  public
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TParityProperty }

  TParityProperty = class(TIntListProperty)
  public
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TStopBitsProperty }

  TStopBitsProperty = class(TIntListProperty)
  public
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TDataBitsProperty }

  TDataBitsProperty = class(TIntListProperty)
  public
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TModemNameProperty }

  TModemNameProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TModemCommandProperty }

  TModemCommandProperty = class(TStringProperty)
  public
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TModemCompressionProperty }

  TModemCompressionProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TModemDeviceProperty }

  TModemDeviceProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

{ TModemPlayDeviceProperty }

  TModemPlayDeviceProperty = class(TModemDeviceProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;


{ TModemRecDeviceProperty }

  TModemRecDeviceProperty = class(TModemDeviceProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ TFileNameProperty }

  TFileNameProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

implementation

uses
  VML_Idents, VML_Consts, VML_Utils, VML_Profile, VML_Log, VML_Port,
  VML_Modem, VML_NoModem, VML_IS101, VML_USRobotics, VML_ZyXEL1496,
  VML_ZyXEL2864, VML_MotorolaSM56, VML_CLMD1414, VML_Rockwell;

{ TIntListProperty }

function TIntListProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable, paReadOnly];
end;

{ TPortProperty }

const
  PortMap: array[0..3] of TIdentMapEntry = (
    (Value: 1; Name: 'COM1'),
    (Value: 2; Name: 'COM2'),
    (Value: 3; Name: 'COM3'),
    (Value: 4; Name: 'COM4'));

function TPortProperty.GetValue: string;
begin
  Result := IntToIdentDef(GetOrdValue, inherited GetValue, PortMap);
end;

procedure TPortProperty.GetValues(Proc: TGetStrProc);
begin
  GetIdentsList(PortMap, Proc);
end;

procedure TPortProperty.SetValue(const Value: string);
begin
  SetOrdValue(IdentToIntDef(Value, GetOrdValue, PortMap));
end;

{ TBaudRateProperty }

const
  BaudRateMap: array[0..14] of TIdentMapEntry = (
    (Value: CBR_110; Name: '110'),
    (Value: CBR_300; Name: '300'),
    (Value: CBR_600; Name: '600'),
    (Value: CBR_1200; Name: '1200'),
    (Value: CBR_2400; Name: '2400'),
    (Value: CBR_4800; Name: '4800'),
    (Value: CBR_9600; Name: '9600'),
    (Value: CBR_14400; Name: '14400'),
    (Value: CBR_19200; Name: '19200'),
    (Value: CBR_38400; Name: '38400'),
    (Value: CBR_56000; Name: '56000'),
    (Value: CBR_57600; Name: '57600'),
    (Value: CBR_115200; Name: '115200'),
    (Value: CBR_128000; Name: '128000'),
    (Value: CBR_256000; Name: '256000'));

function TBaudRateProperty.GetValue: string;
begin
  Result := IntToIdentDef(GetOrdValue, inherited GetValue, BaudRateMap);
end;

procedure TBaudRateProperty.GetValues(Proc: TGetStrProc);
begin
  GetIdentsList(BaudRateMap, Proc);
end;

procedure TBaudRateProperty.SetValue(const Value: string);
begin
  SetOrdValue(IdentToIntDef(Value, GetOrdValue, BaudRateMap));
end;

{ TParityProperty }

const
  ParityMap: array[0..4] of TIdentMapEntry = (
    (Value: NOPARITY; Name: 'None'),
    (Value: ODDPARITY; Name: 'Odd'),
    (Value: EVENPARITY; Name: 'Even'),
    (Value: MARKPARITY; Name: 'Mark'),
    (Value: SPACEPARITY; Name: 'Space'));

function TParityProperty.GetValue: string;
begin
  Result := IntToIdentDef(GetOrdValue, inherited GetValue, ParityMap);
end;

procedure TParityProperty.GetValues(Proc: TGetStrProc);
begin
  GetIdentsList(ParityMap, Proc);
end;

procedure TParityProperty.SetValue(const Value: string);
begin
  SetOrdValue(IdentToIntDef(Value, GetOrdValue, ParityMap));
end;

{ TStopBitsProperty }

const
  StopBitsMap: array[0..2] of TIdentMapEntry = (
    (Value: ONESTOPBIT; Name: '1'),
    (Value: ONE5STOPBITS; Name: '1.5'),
    (Value: TWOSTOPBITS; Name: '2'));

function TStopBitsProperty.GetValue: string;
begin
  Result := IntToIdentDef(GetOrdValue, inherited GetValue, StopBitsMap);
end;

procedure TStopBitsProperty.GetValues(Proc: TGetStrProc);
begin
  GetIdentsList(StopBitsMap, Proc);
end;

procedure TStopBitsProperty.SetValue(const Value: string);
begin
  SetOrdValue(IdentToIntDef(Value, GetOrdValue, StopBitsMap));
end;

{ TDataBitsProperty }

const
  DataBitsMap: array[0..4] of TIdentMapEntry = (
    (Value: 4; Name: '4'),
    (Value: 5; Name: '5'),
    (Value: 6; Name: '6'),
    (Value: 7; Name: '7'),
    (Value: 8; Name: '8'));

function TDataBitsProperty.GetValue: string;
begin
  Result := IntToIdentDef(GetOrdValue, inherited GetValue, DataBitsMap);
end;

procedure TDataBitsProperty.GetValues(Proc: TGetStrProc);
begin
  GetIdentsList(DataBitsMap, Proc);
end;

procedure TDataBitsProperty.SetValue(const Value: string);
begin
  SetOrdValue(IdentToIntDef(Value, GetOrdValue, DataBitsMap));
end;

{ TModemNameProperty }

function TModemNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

procedure TModemNameProperty.GetValues(Proc: TGetStrProc);
begin
  GetModemDriversList(Proc);
end;

{ TModemCommandProperty }

type
  TCharListEntry = record
    Value: Char;
    Name1: string;
    Name2: string;
  end;

  PCharList = ^TCharList;
  TCharList = array[0..0] of TCharListEntry;

const
  ControlChars: array[0..33] of TCharListEntry = (
    (Value: NUL; Name1: '<NUL>'; Name2: '^@'),
    (Value: SOH; Name1: '<SOH>'; Name2: '^A'),
    (Value: STX; Name1: '<STX>'; Name2: '^B'),
    (Value: ETX; Name1: '<ETX>'; Name2: '^C'),
    (Value: EOT; Name1: '<EOT>'; Name2: '^D'),
    (Value: ENQ; Name1: '<ENQ>'; Name2: '^E'),
    (Value: ACK; Name1: '<ACK>'; Name2: '^F'),
    (Value: BEL; Name1: '<BEL>'; Name2: '^G'),
    (Value: BS; Name1: '<BS>'; Name2: '^H'),
    (Value: HT; Name1: '<HT>'; Name2: '^I'),
    (Value: LF; Name1: '<LF>'; Name2: '^J'),
    (Value: VT; Name1: '<VT>'; Name2: '^K'),
    (Value: FF; Name1: '<FF>'; Name2: '^L'),
    (Value: CR; Name1: '<CR>'; Name2: '^M'),
    (Value: SO; Name1: '<SO>'; Name2: '^N'),
    (Value: SI; Name1: '<SI>'; Name2: '^O'),
    (Value: DLE; Name1: '<DLE>'; Name2: '^P'),
    (Value: DC1; Name1: '<DC1>'; Name2: '^Q'),
    (Value: DC2; Name1: '<DC2>'; Name2: '^R'),
    (Value: DC3; Name1: '<DC3>'; Name2: '^S'),
    (Value: DC4; Name1: '<DC4>'; Name2: '^T'),
    (Value: NAK; Name1: '<NAK>'; Name2: '^U'),
    (Value: SYN; Name1: '<SYN>'; Name2: '^V'),
    (Value: ETB; Name1: '<ETB>'; Name2: '^W'),
    (Value: CAN; Name1: '<CAN>'; Name2: '^X'),
    (Value: EM; Name1: '<EM>'; Name2: '^Y'),
    (Value: SUB; Name1: '<SUB>'; Name2: '^Z'),
    (Value: ESC; Name1: '<ESC>'; Name2: '^['),
    (Value: FS; Name1: '<FS>'; Name2: '^\'),
    (Value: GS; Name1: '<GS>'; Name2: '^]'),
    (Value: RS; Name1: '<RS>'; Name2: '^^'),
    (Value: US; Name1: '<US>'; Name2: '^_'),
    (Value: XON; Name1: '<XON>'; Name2: '^Q'),
    (Value: XOFF; Name1: '<XOFF>'; Name2: '^S'));

function TModemCommandProperty.GetValue: string;
var
  S: string;
  I: Integer;

  function ReplaceChar(C: Char): string;
  var
    I: Integer;
  begin
    { Find replacement for standard control char }
    for I := 0 to High(ControlChars) do
      if C = ControlChars[I].Value then
      begin
        Result := ControlChars[I].Name1;
        Exit;
      end;
    { Given char is not control char }
    Result := C;
  end;

begin
  { Get value }
  S := inherited GetValue;
  { Convert control chars to cpecial sequence }
  Result := '';
  for I := 1 to Length(S) do
    if S[I] >= ' ' then
      Result := Result + S[I]
    else
      Result := Result + ReplaceChar(S[I]);
end;

procedure TModemCommandProperty.SetValue(const Value: string);
var
  I: Integer;
  S: string;

  procedure ReplaceSequence(Sequence: string; C: Char);
  var
    P: Integer;
  begin
    { Find first entry point }
    P := Pos(Sequence, S);
    { Replace all sequences }
    while P > 0 do
    begin
      Delete(S, P, Length(Sequence));
      Insert(C, S, P);
      { Find next sequence entry point }
      P := Pos(Sequence, S);
    end;
  end;

begin
  { Replace control sequences }
  S := Value;
  for I := 0 to High(ControlChars) do
  begin
    ReplaceSequence(ControlChars[I].Name1, ControlChars[I].Value);
    ReplaceSequence(ControlChars[I].Name2, ControlChars[I].Value);
  end;
  { Set value }
  inherited SetValue(S);
end;

{ TModemCompressionProperty }

function TModemCompressionProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TModemCompressionProperty.GetValue: string;
begin
  Result := GetCompressionName(TModemProfile(GetComponent(0)).ModemName, GetOrdValue);
end;

procedure TModemCompressionProperty.GetValues(Proc: TGetStrProc);
begin
  GetCompressionsList(TModemProfile(GetComponent(0)).ModemName, Proc);
end;

procedure TModemCompressionProperty.SetValue(const Value: string);
begin
  SetOrdValue(GetCompression(TModemProfile(GetComponent(0)).ModemName, Value));
end;

{ TModemDeviceProperty }

function TModemDeviceProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paRevertable];
end;

function TModemDeviceProperty.GetValue: string;
begin
  Result := GetDeviceName(TModemProfile(GetComponent(0)).ModemName, GetOrdValue);
end;

procedure TModemDeviceProperty.SetValue(const Value: string);
begin
  SetOrdValue(GetDevice(TModemProfile(GetComponent(0)).ModemName, Value));
end;

{ TModemPlayDeviceProperty }

procedure TModemPlayDeviceProperty.GetValues(Proc: TGetStrProc);
begin
  GetPlayDevicesList(TModemProfile(GetComponent(0)).ModemName, Proc);
end;

{ TModemRecDeviceProperty }

procedure TModemRecDeviceProperty.GetValues(Proc: TGetStrProc);
begin
  GetRecDevicesList(TModemProfile(GetComponent(0)).ModemName, Proc);
end;

{ TFileNameProperty }

procedure TFileNameProperty.Edit;
begin
  with TOpenDialog.Create(nil) do
  try
    DefaultExt := 'log';
    FileName := GetValue;
    Filter := 'Log Files (*.log)|*.log|All Files (*.*)|*.*';
    Options := [ofHideReadOnly, ofPathMustExist];
    if Execute then SetValue(FileName);
  finally
    Free;
  end;
end;

function TFileNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;

procedure Register;
begin
  RegisterComponents('VML', [TVoiceModem]);
  RegisterComponents('VML', [TModemProfile]);
  RegisterComponents('VML', [TModemLogFile]);
  RegisterComponents('VML', [TModemPort]);

  RegisterComponents('VML', [TNoModem]);
  RegisterComponents('VML', [TIS101]);
  RegisterComponents('VML', [TUSRobotics]);
  RegisterComponents('VML', [TZyXEL1496]);
  RegisterComponents('VML', [TZyXEL2864]);
  RegisterComponents('VML', [TMotorolaSM56]);
  RegisterComponents('VML', [TCirrusLogicMD1414]);
  RegisterComponents('VML', [TRockwell]);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'Port', TPortProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortBaudRate', TBaudRateProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortParity', TParityProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortStopBits', TStopBitsProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PortDataBits', TDataBitsProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'RecCompression', TModemCompressionProperty);

  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'RecDevice', TModemRecDeviceProperty);
  RegisterPropertyEditor(TypeInfo(Integer), TModemProfile, 'PlayDevice', TModemPlayDeviceProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemProfile, 'ModemName', TModemNameProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemLogFile, 'FileName', TFileNameProperty);

  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'PickPhoneCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'PickPhoneAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BeepCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BeepAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'BeepTimeUnit', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'HardFlowCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'HardFlowAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SoftFlowCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SoftFlowAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartPlayAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'ResetPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'IntrPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'IntrPlayAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopPlayCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopPlayAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartRecCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StartRecAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopRecCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'StopRecAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SwitchModeCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'SwitchModeAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOnCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOnAnsw', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOffCmd', TModemCommandProperty);
  RegisterPropertyEditor(TypeInfo(string), TModemDriver, 'VoiceModeOffAnsw', TModemCommandProperty);
end;

end.
